// See COPYING for license information.

#include <algorithm>
#include "defs.h"
#include "CParser.h"
#include "CConfigFile.h"
#include "CPkgRemover.h"

CPkgRemover::CPkgRemover(std::vector<std::string>& files, const bool& verbose)
{
	// Ignore case when a filename was not specified.
	CParser	_parser;
	for ( unsigned int i = 0; i < files.size(); i++ ) {
		if ( files[i].find(".t") != files[i].length() - 4 || files[i].length() == 3 ) {
			std::transform(files[i].begin(), files[i].end(), files[i].begin(), tolower);
			if ( !_parser.Packages[files[i]].name.empty() )
				files[i] = _parser.Packages[files[i]].name;
		}
	}

	// Get the blacklist from the config file.
	std::string	_blacklist = CConfigFile::Instance().GetValue("BlackList")[0];
	std::transform(_blacklist.begin(), _blacklist.end(), _blacklist.begin(), tolower);
	
	// Remove all the remaining packages in the list.
	for ( unsigned int i = 0; i < files.size(); i++ ) {
		_File = files[i].substr((int)files[i].rfind('/') + 1);
		std::transform(_File.begin(), _File.end(), _File.begin(), tolower);

		if ( (int)_blacklist.find("," + _File + ",") >= 0 ) {
			std::cout << "Found package '" << _File << "' on the blacklist, skipping.\n";
			continue;
		}

		std::cout << "Removing (" << i + 1 << " of " << files.size() << "): " << _File << "...";
		std::cout.flush();
		WRITELOG("Removing package: " + _File);
		std::string	_cmd = "rm -f " + TMPFILE(_File + ".done");
		system(_cmd.c_str());
		if ( verbose )	_cmd = "/sbin/removepkg " + files[i] + " && touch " + TMPFILE(_File + ".done") + " &";
		else	_cmd = "/sbin/removepkg " + files[i] + " > /dev/null && touch " + TMPFILE(_File + ".done") + " &";
		system(_cmd.c_str());
		while ( iteration() == 0.0f ) {}
		std::cout << "done\n";
		_cmd = "rm -f " + TMPFILE(_File + ".done");
		system(_cmd.c_str());
	}
}

CPkgRemover::~CPkgRemover()
{
	std::string	_cmd = "rm -f " + TMPFILE(_File + ".done");
	system(_cmd.c_str());
}

float CPkgRemover::iteration()
{	// Call this periodically to determine the progress of removal.
	float		_status = 0.0f;
	std::ifstream	_if;
	_if.open(TMPFILE(_File + ".done").c_str(), std::ios_base::in);
	if ( _if.is_open() ) {
		WRITELOG("Finished removing: " + _File);
		_status = 2.0f;
	}
	_if.close();
	return _status;
}
